package puma.database;

import java.util.NoSuchElementException;
import java.util.Vector;

/*import DatenbankKern;
import DatenbankException;
import Liste;*/

public class Operationen {

  private static Operationen ops = null;
  private final static String personVorname = "VORNAME";
  private final static String personNachname = "NAME";
  private final static String personID = "PERSON_ID";
  private final static String personPasswort = "PASSWORT";
  private DatenbankKern DBMS = null;

  private Operationen() {
    // Datenbank Initialisieren fuer Operations-Objekt
    DBMS = DatenbankKern.getInstance();
  }

  // Bindet ein Datenbanksystem an einen Operationen-Satz
  public void verbinden(DatenbankKern neuesDBMS) {
    DBMS = neuesDBMS;
  }

  // Legt neue Instanz bei bedarf an
  public static Operationen getInstance() {
    if (ops == null) ops = new Operationen();
    return ops;
  }

  // holt die PersonID von einer eindeutig identifizierten Person
  public int holePersonIDvon(String Benutzername) throws OperationException {
    int position = Benutzername.lastIndexOf(' ');
    Liste suchergebnis;
    Liste schluessel = new Liste();
    StringDaten schluesselVorname;
    StringDaten schluesselNachname;

    if (position == 0) {
      schluesselVorname = new StringDaten(personVorname,Benutzername.trim());
      schluessel.Anfuegen(schluesselVorname);
    } else {
      schluesselVorname = new StringDaten(
          personVorname,
          Benutzername.substring(0,position).trim());
      schluesselNachname = new StringDaten(
          personNachname,
          Benutzername.substring(position+1));
      schluessel.Anfuegen(schluesselVorname);
      schluessel.Anfuegen(schluesselNachname);
    }
    try {
      suchergebnis = DBMS.PersonSuchen(schluessel);
      return Integer.parseInt(suchergebnis.Suchen(personID).Inhalt());
    } catch (NumberFormatException e) {
      throw new OperationException("(holePersonIDvon) Interner Fehler : Person hat keine Integer-ID !");
    } catch (NoSuchElementException e) {
      throw new OperationException("(holePersonIDvon) Interner Fehler : Person hat berhaupt keine ID !");
    } catch (DatenbankException e) {
      throw new OperationException("(holePersonIDvon) Datenbank-Fehler : "+e.toString());
    }
  }

  // Zugriffrechte holen
  public StringListeDaten holeZGR(int userID) throws OperationException {
    try {
      return DBMS.HoleZGR(userID);
    } catch (DatenbankException e) {
      throw new OperationException("(holeZGR) Datenbank-Fehler : "+e.toString());
    }
  }

  // Ausleihliste holen
  public Liste getAusleihliste(int userID) throws OperationException {
    try {
      return DBMS.PersonAusleihliste(userID);
    } catch (DatenbankException e) {
      throw new OperationException("(getAusleihliste) Datenbank-Fehler : "+e.toString());
    }
  }

  // Publikation suchen
  public Liste suche_Publ(Liste schluessel) throws OperationException {
    try {
      return DBMS.PublSuchen(schluessel);
    } catch (DatenbankException e) {
      throw new OperationException("(suche_Publ) Datenbank-Fehler : "+e.toString());
    }
  }

  // Instanzstatus aendern mit Integer-Inventarnummer
  public void instanzStatusAendern(int inventarnummer, String neuerStatus, int personID) throws OperationException {
    try {
      DBMS.InstanzStatusAendern(Integer.toString(inventarnummer),personID,neuerStatus);
    } catch (DatenbankException e) {
      throw new OperationException("(instanzStatusAendern.1) Datenbank-Fehler : "+e.toString());
    }
  }

  // Instanzstatus aendern aufgrund von einer Inventarnummer als
  // String
  public void instanzStatusAendern(String inventarnummer, String neuerStatus, int personID) throws OperationException {
    try {
      DBMS.InstanzStatusAendern(inventarnummer,personID,neuerStatus);
    } catch (DatenbankException e) {
      throw new OperationException("(instanzStatusAendern.2) Datenbank-Fehler : "+e.toString());
    }
  }

  // besorgt die Instanzen einer bestimmten Publikation
  public Liste instanzenVonPubl(String signatur) throws OperationException {
    try {
      return DBMS.InstanzenVonPubl(signatur);
    } catch (DatenbankException e) {
      throw new OperationException("(instanzenVonPubl) Datenbank-Fehler : "+e.toString());
    }
  }

  // Publikation suchen und als Vector von Listen zurueckgeben
  public Vector PubSuchen(Liste schluessel) throws OperationException {
    try {
      Liste suchergebnis = DBMS.PublSuchen(schluessel);
      int anzahlBlocks = 0;
      int count;
      Vector ergebnis = new Vector();

      // Bloecke zaehlen
      for (count = 0; count < suchergebnis.Anzahl(); count++)
        if (suchergebnis.Element(count).Inhalt() == null) anzahlBlocks++;

      while (anzahlBlocks-- > 0)
        ergebnis.addElement(suchergebnis.ExtrahiereBlock(anzahlBlocks));
      return ergebnis;

    } catch (DatenbankException e) {
      throw new OperationException("(PubSuchen) Datenbank-Fehler"+e.toString());
    } catch (ArrayIndexOutOfBoundsException e) {
      throw new OperationException("(PubSuchen) Fehler : Zu viele Elemente gelesen");
    }
  }

  // Passwort ueberpruefen
  public static boolean checkPassword(String UName, String Password) throws OperationException {
    int position = UName.lastIndexOf(' ');
    Liste suchergebnis;
    Liste schluessel = new Liste();
    StringDaten schluesselVorname;
    StringDaten schluesselNachname;

    if (position == 0) {
      schluesselVorname = new StringDaten(personVorname,UName.trim());
      schluessel.Anfuegen(schluesselVorname);
    } else {
      schluesselVorname = new StringDaten(
          personVorname,
          UName.substring(0,position).trim());
      schluesselNachname = new StringDaten(
          personNachname,
          UName.substring(position+1));
      schluessel.Anfuegen(schluesselVorname);
      schluessel.Anfuegen(schluesselNachname);
    }
    try {
      suchergebnis = DatenbankKern.getInstance().PersonSuchen(schluessel);
      return suchergebnis.Suchen(personPasswort).Inhalt().equals(Password);
    } catch (NoSuchElementException e) {
      throw new OperationException("(checkPassword) Interner Fehler : Person hat kein Passwort !");
    } catch (DatenbankException e) {
      throw new OperationException("(checkPassword) Datenbank-Fehler : "+e.toString());
    }
  }
  
  // Alle Benutzernamen liefern
  public StringListeDaten holeBenutzerListe() {
     Liste query = new Liste();
     query.Anfuegen(new StringDaten("NAME","%"));
     Liste allPersons = null;
     try {
        allPersons =
          DBMS.PersonSuchen(query);
     } catch (DatenbankException e) {return null;}
     
     StringListeDaten result = new StringListeDaten("");     
     try {
        Datenobjekt aktuell = allPersons.ErstesElement();     
	String Vorname = "";
	String Name = "";
	
        while (true) {
	   if (aktuell.Beschreibung() == null) {
	      result.Anfuegen(new String(Name+", "+Vorname));
	      Vorname = "";
	      Name = "";
	   }	   
	   if (aktuell.Beschreibung().equals("NAME") )
	      Name = aktuell.Inhalt();
	   if (aktuell.Beschreibung().equals("VORNAME") )
	      Vorname = aktuell.Inhalt();
	   aktuell = allPersons.NaechstesElement();
        }
     } catch (NoSuchElementException e) { }	
     return result;     
  }
}
