package puma.control;

import puma.database.*;

/** Command, das eine Ausleihliste anfordert.
 *  @author Sven Mller
 */
public class AusleihlisteCmd extends Command {
      
   /** Standard-Konstruktor.
    * @param serverApp  Ein Rckverweis auf die aufrufende Server-Instanz.
    */      
   public AusleihlisteCmd(ServerApplication serverApp) {
      super(serverApp);
   }      
      
      
   /** Fhrt die dem Command zugeordnete Aktion aus.
    *  
    * Es wird ein neues Objekt vom Typ Ausleihliste erzeugt. Diesem wird im
    * Konstruktor ein Benutzername bergeben.
    * Der Benutzername kann entweder ein beliebiger sein, der von <i>in</i>
    * zurckgeliefert wird, oder (falls <i>in</i> keinen Benutzer kennt) der
    * Client-User, welcher via Instanzvariablen serverApp ermittelt wird.
    * Mit <i>out</i> wird das Ergebnis in das Ausgabeformat gebracht.
    *
    * @see Ausleihliste
    * @param in Objekt, das Input-Daten liefert.
    * @param out Objekt, das fr die Darstellung der Output-Daten sorgt.
    */      
   public void execute(InType in, OutType out) {
      try {
         // Bezueglich (anderem) Ausleiher oder Owner!
         int userID = in.getAusleiher();
         if (userID == 0)
             userID = serverApp.getUserID();
         Ausleihliste searchObj = new Ausleihliste(userID);
         out.setAusleihliste(
             searchObj.deliverResult(),
	         Operationen.getInstance().holeBenutzerListe() );
      } catch (Exception e) { out.createErrorMsg(e); }
   }   

}
